@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2004 Kannel Group  
+ * Copyright (c) 2001-2009 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -209,7 +209,7 @@ enum { P_INT, P_STRING, P_ADDRESS, P_TIME, P_HEX, P_SMS };
  * outgoing messages. */
 static const struct
 {
-    unsigned char *name;
+    char *name;
     int number;
     int maxlen;
     int type;  /* P_ values */
@@ -257,7 +257,7 @@ parameters[] = {
 
 /* Return the index in the parameters array for this parameter id.
  * Return -1 if it is not found. */
-static const int parm_index(int parmno)
+static int parm_index(int parmno)
 {
     int i;
 
@@ -271,7 +271,7 @@ static const int parm_index(int parmno)
 
 #ifndef NO_GWASSERT
 /* Return the type of this parameter id.  Return -1 if the id is unknown. */
-static const int parm_type(int parmno)
+static int parm_type(int parmno)
 {
     int i = parm_index(parmno);
 
@@ -284,7 +284,7 @@ static const int parm_type(int parmno)
 
 /* Return the max length for this parameter id.
  * Return -1 if the id is unknown. */
-static const int parm_maxlen(int parmno)
+static int parm_maxlen(int parmno)
 {
     int i = parm_index(parmno);
 
@@ -307,7 +307,7 @@ static const char *parm_name(int parmno)
 #ifndef NO_GWASSERT
 /* Return 1 if the value for this (Integer) parameter is in range.
  * Return 0 otherwise.  Return -1 if the parameter was not found.  */
-static const int parm_in_range(int parmno, long value)
+static int parm_in_range(int parmno, long value)
 {
     int i;
 
@@ -326,7 +326,7 @@ static int isphonedigit(int c)
     return isdigit(c) || c == '+' || c == '-';
 }
 
-static const int parm_valid_address(Octstr *value)
+static int parm_valid_address(Octstr *value)
 {
     return octstr_check_range(value, 0, octstr_len(value), isphonedigit);
 }
@@ -337,12 +337,12 @@ static const int parm_valid_address(Octstr *value)
 
 static int operation_find(int operation);
 static Octstr *operation_name(int operation);
-static const int operation_can_send(int operation);
-static const int operation_can_receive(int operation);
+static int operation_can_send(int operation);
+static int operation_can_receive(int operation);
 
 static const struct
 {
-    unsigned char *name;
+    char *name;
     int code;
     int can_send;
     int can_receive;
@@ -403,7 +403,7 @@ static Octstr *operation_name(int operation)
 }
 
 /* Return true if a CIMD2 client may send this operation */
-static const int operation_can_send(int operation)
+static int operation_can_send(int operation)
 {
     int i = operation_find(operation);
 
@@ -419,7 +419,7 @@ static const int operation_can_send(int operation)
 
 
 /* Return true if a CIMD2 server may send this operation */
-static const int operation_can_receive(int operation)
+static int operation_can_receive(int operation)
 {
     int i = operation_find(operation);
 
@@ -919,7 +919,7 @@ static void packet_check_can_receive(struct packet *packet, SMSCConn *conn)
 static struct
 {
     int code;
-    unsigned char *text;
+    char *text;
 }
 cimd2_errors[] = {
     { 0, "No error" },
@@ -1177,7 +1177,7 @@ static void convert_gsm_to_cimd2(Octstr *text)
 static struct packet *packet_create(int operation, int seq)
 {
     struct packet *packet;
-    unsigned char minpacket[sizeof("sOO:SSSte")];
+    char minpacket[sizeof("sOO:SSSte")];
 
     packet = gw_malloc(sizeof(*packet));
     packet->operation = operation;
@@ -1192,7 +1192,7 @@ static struct packet *packet_create(int operation, int seq)
 static void packet_add_parm(struct packet *packet, int parmtype,
                             int parmno, Octstr *value, SMSCConn *conn)
 {
-    unsigned char parmh[sizeof("tPPP:")];
+    char parmh[sizeof("tPPP:")];
     long position;
     long len;
     int copied = 0;
@@ -1261,7 +1261,7 @@ static void packet_add_hex_parm(struct packet *packet, int parmno, Octstr *value
 /* Add an Integer parameter to the packet */
 static void packet_add_int_parm(struct packet *packet, int parmno, long value, SMSCConn *conn)
 {
-    unsigned char buf[128];
+    char buf[128];
     Octstr *valuestr;
 
     gw_assert(parm_in_range(parmno, value));
@@ -1277,7 +1277,7 @@ static void packet_set_checksum(struct packet *packet)
     Octstr *data;
     int checksum;
     long pos, len;
-    unsigned char buf[16];
+    char buf[16];
 
     gw_assert(packet != NULL);
 
@@ -1302,7 +1302,7 @@ static void packet_set_checksum(struct packet *packet)
 
 static void packet_set_sequence(struct packet *packet, int seq)
 {
-    unsigned char buf[16];
+    char buf[16];
 
     gw_assert(packet != NULL);
     gw_assert(seq >= 0);
@@ -1484,7 +1484,7 @@ static struct packet *packet_encode_message(Msg *msg, Octstr *sender_prefix, SMS
         /* Going from latin1 to GSM to CIMD2 may seem like a
          * detour, but it's the only way to get all the escape
          * codes right. */
-        charset_latin1_to_gsm(text);
+        charset_utf8_to_gsm(text);
         truncated = charset_gsm_truncate(text, spaceleft);
         convert_gsm_to_cimd2(text);
         packet_add_sms_parm(packet, P_USER_DATA, text, conn);
@@ -1598,7 +1598,7 @@ static Msg *cimd2_accept_message(struct packet *request, SMSCConn *conn)
      *   000xyyxx  Uncompressed text, yy indicates alphabet.
      *                   yy = 00, default alphabet
      *                   yy = 01, 8-bit data
-     *                   yy = 10, UCS2
+     *                   yy = 10, UCS-2
      *                   yy = 11, reserved
      *   1111xyxx  Data, y indicates alphabet.
      *                   y = 0, default alphabet
@@ -1613,7 +1613,7 @@ static Msg *cimd2_accept_message(struct packet *request, SMSCConn *conn)
     text = packet_get_sms_parm(request, P_USER_DATA);
     if (text != NULL) {
         convert_cimd2_to_gsm(text,conn);
-        charset_gsm_to_latin1(text);
+        charset_gsm_to_utf8(text);
     } else {
         /*
          * FIXME: If DCS indicates GSM charset, and we get it in binary,
@@ -1689,12 +1689,12 @@ static void cimd2_handle_request(struct packet *request, SMSCConn *conn)
         if (request->operation == DELIVER_STATUS_REPORT) {
             message = cimd2_accept_delivery_report_message(request, conn);
             if (message)
-                list_append(pdata->received, message);
+                gwlist_append(pdata->received, message);
         }
         else if (request->operation == DELIVER_MESSAGE) {
             message = cimd2_accept_message(request,conn);
             if (message)
-                list_append(pdata->received, message);
+                gwlist_append(pdata->received, message);
         }
     }
 
@@ -1939,15 +1939,15 @@ static void cimd2_destroy(PrivData *pdata)
     octstr_destroy(pdata->inbuffer);
     octstr_destroy(pdata->my_number);
 
-    discarded = list_len(pdata->received);
+    discarded = gwlist_len(pdata->received);
     if (discarded > 0)
         warning(0, "CIMD2[%s]: discarded %d received messages",
                 octstr_get_cstr(pdata->conn->id), 
                 discarded);
 
-    list_destroy(pdata->received, msg_destroy_item);
-    list_destroy(pdata->outgoing_queue, NULL);
-    list_destroy(pdata->stopped, NULL);
+    gwlist_destroy(pdata->received, msg_destroy_item);
+    gwlist_destroy(pdata->outgoing_queue, NULL);
+    gwlist_destroy(pdata->stopped, NULL);
 
     gw_free(pdata);
 }
@@ -2007,8 +2007,8 @@ static int cimd2_receive_msg(SMSCConn *conn, Msg **msg)
 
     gw_assert(pdata != NULL);
 
-    if (list_len(pdata->received) > 0) {
-        *msg = list_consume(pdata->received);
+    if (gwlist_len(pdata->received) > 0) {
+        *msg = gwlist_consume(pdata->received);
         return 1;
     }
 
@@ -2072,8 +2072,8 @@ static int cimd2_receive_msg(SMSCConn *conn, Msg **msg)
         packet_destroy(packet);
     }
 
-    if (list_len(pdata->received) > 0) {
-	*msg = list_consume(pdata->received);
+    if (gwlist_len(pdata->received) > 0) {
+	*msg = gwlist_consume(pdata->received);
         return 1;
     }
     return 0;
@@ -2172,7 +2172,7 @@ static void io_thread (void *arg)
     /* remove messages from SMSC until we are killed */
     while (!pdata->quitting) {
     
-        list_consume(pdata->stopped); /* block here if suspended/isolated */
+        gwlist_consume(pdata->stopped); /* block here if suspended/isolated */
       
         /* check that connection is active */
         if (conn->status != SMSCCONN_ACTIVE) {
@@ -2206,7 +2206,7 @@ static void io_thread (void *arg)
  
         /* send messages */
         do {
-            msg = list_extract_first(pdata->outgoing_queue);
+            msg = gwlist_extract_first(pdata->outgoing_queue);
             if (msg) {
                 sleep = 0;
                 if (cimd2_submit_msg(conn,msg) != 0) break;
@@ -2240,7 +2240,7 @@ static int cimd2_add_msg_cb (SMSCConn *conn, Msg *sms)
     Msg *copy;
 
     copy = msg_duplicate(sms);
-    list_produce(pdata->outgoing_queue, copy);
+    gwlist_produce(pdata->outgoing_queue, copy);
     gwthread_wakeup(pdata->io_thread);
 
     return 0;
@@ -2263,14 +2263,14 @@ static int cimd2_shutdown_cb (SMSCConn *conn, int finish_sending)
 
     if (finish_sending == 0) {
         Msg *msg;
-        while ((msg = list_extract_first(pdata->outgoing_queue)) != NULL) {
+        while ((msg = gwlist_extract_first(pdata->outgoing_queue)) != NULL) {
             bb_smscconn_send_failed(conn, msg, SMSCCONN_FAILED_SHUTDOWN, NULL);
         }
     }
 
     cimd2_logout(conn);
     if (conn->is_stopped) {
-        list_remove_producer(pdata->stopped);
+        gwlist_remove_producer(pdata->stopped);
         conn->is_stopped = 0;
     }
 
@@ -2293,7 +2293,7 @@ static void cimd2_start_cb (SMSCConn *conn)
 {
     PrivData *pdata = conn->data;
 
-    list_remove_producer(pdata->stopped);
+    gwlist_remove_producer(pdata->stopped);
     /* in case there are messages in the buffer already */
     gwthread_wakeup(pdata->io_thread);
     debug("bb.sms", 0, "SMSCConn CIMD2 %s, start called",
@@ -2303,7 +2303,7 @@ static void cimd2_start_cb (SMSCConn *conn)
 static void cimd2_stop_cb (SMSCConn *conn)
 {
     PrivData *pdata = conn->data;
-    list_add_producer(pdata->stopped);
+    gwlist_add_producer(pdata->stopped);
     debug("bb.sms", 0, "SMSCConn CIMD2 %s, stop called",
           octstr_get_cstr(conn->id));
 }
@@ -2312,7 +2312,7 @@ static long cimd2_queued_cb (SMSCConn *conn)
 {
     PrivData *pdata = conn->data;
     conn->load = (pdata ? (conn->status != SMSCCONN_DEAD ? 
-                  list_len(pdata->outgoing_queue) : 0) : 0);
+                  gwlist_len(pdata->outgoing_queue) : 0) : 0);
     return conn->load; 
 }
 
@@ -2329,16 +2329,16 @@ int smsc_cimd2_create(SMSCConn *conn, CfgGroup *grp)
     pdata->no_dlr = 0;
     pdata->quitting = 0;
     pdata->socket = -1;
-    pdata->received = list_create();
+    pdata->received = gwlist_create();
     pdata->inbuffer = octstr_create("");
     pdata->send_seq = 1;
     pdata->receive_seq = 0;
-    pdata->outgoing_queue = list_create();
-    pdata->stopped = list_create();
-    list_add_producer(pdata->outgoing_queue);
+    pdata->outgoing_queue = gwlist_create();
+    pdata->stopped = gwlist_create();
+    gwlist_add_producer(pdata->outgoing_queue);
 
     if (conn->is_stopped)
-      list_add_producer(pdata->stopped);
+      gwlist_add_producer(pdata->stopped);
 
     pdata->host = cfg_get(grp, octstr_imm("host"));
     if (cfg_get_integer(&(pdata->port), grp, octstr_imm("port")) == -1)
